import { List, Map } from "immutable";
import { StringMap } from "./Support";
import { TypeGraphBuilder } from "./TypeBuilder";
export declare enum PathElementKind {
    Root = 0,
    KeyOrIndex = 1,
    Type = 2,
    Object = 3,
}
export declare type PathElement = {
    kind: PathElementKind.Root;
} | {
    kind: PathElementKind.KeyOrIndex;
    key: string;
} | {
    kind: PathElementKind.Type;
    index: number;
} | {
    kind: PathElementKind.Object;
};
export declare type JSONSchema = StringMap | boolean;
export declare function checkJSONSchema(x: any): JSONSchema;
export declare class Ref {
    readonly path: List<PathElement>;
    static root(address: string): Ref;
    private static parsePath(path);
    static parse(ref: any): Ref;
    addressURI: uri.URI | undefined;
    constructor(addressURI: uri.URI | undefined, path: List<PathElement>);
    readonly hasAddress: boolean;
    readonly address: string;
    private pushElement(pe);
    push(...keys: string[]): Ref;
    pushObject(): Ref;
    pushType(index: number): Ref;
    resolveAgainst(base: Ref | undefined): Ref;
    readonly name: string;
    readonly definitionName: string | undefined;
    toString(): string;
    lookupRef(root: JSONSchema): JSONSchema;
    equals(other: any): boolean;
    hashCode(): number;
}
export declare abstract class JSONSchemaStore {
    private _schemas;
    private add(address, schema);
    abstract fetch(_address: string): Promise<JSONSchema | undefined>;
    get(address: string): Promise<JSONSchema>;
}
export declare function addTypesInSchema(typeBuilder: TypeGraphBuilder, store: JSONSchemaStore, references: Map<string, Ref>): Promise<void>;
export declare function definitionRefsInSchema(store: JSONSchemaStore, address: string): Promise<Map<string, Ref>>;
